import axios from '@/utils/axios'
import { type AxiosResponse } from 'axios'
import {
  LoginInputDto,
  RegisterInputDto,
  ForgetPasswordInputDto,
  DeleteUser,
  Token,
  TokenRefreshPayload,
  User,
} from './user.interface'

export async function register(data: RegisterInputDto) {
  return axios.post<Token>('/api/users', data)
}

export async function login(data: LoginInputDto) {
  return axios.post<Token>('/api/users/token', data)
}

export async function logout() {
  return axios.delete('/api/users/me/token')
}

export async function forgetPassword(data: ForgetPasswordInputDto) {
  return axios.patch<Token>('/api/users/password', data)
}

export async function deleteUser(data: DeleteUser) {
  return axios.delete('/api/users/me', { data })
}

let refreshing: Promise<AxiosResponse<Token>> | null
export async function refreshToken(data: TokenRefreshPayload) {
  if (!refreshing) {
    refreshing = axios.put<Token>('/api/users/me/token', data).finally(() => {
      refreshing = null
    })
  }
  return refreshing
}

export async function getUserInfo() {
  return axios.get<User>('/api/users/me')
}
